"use strict";
/*
 * Copyright (c) 2008-2021, Hazelcast, Inc. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/** @ignore */ /** */
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConfigBuilder = void 0;
const core_1 = require("../core");
const proxy_1 = require("../proxy");
const Util_1 = require("../util/Util");
const Config_1 = require("./Config");
const EvictionPolicy_1 = require("./EvictionPolicy");
const FlakeIdGeneratorConfig_1 = require("./FlakeIdGeneratorConfig");
const InMemoryFormat_1 = require("./InMemoryFormat");
const NearCacheConfig_1 = require("./NearCacheConfig");
const ReliableTopicConfig_1 = require("./ReliableTopicConfig");
const JsonStringDeserializationPolicy_1 = require("./JsonStringDeserializationPolicy");
const ConnectionStrategyConfig_1 = require("./ConnectionStrategyConfig");
const LoadBalancerConfig_1 = require("./LoadBalancerConfig");
const logging_1 = require("../logging");
const security_1 = require("../security");
/**
 * Responsible for user-defined config validation. Builds the effective config with necessary defaults.
 * @internal
 */
class ConfigBuilder {
    constructor(config) {
        this.effectiveConfig = new Config_1.ClientConfigImpl();
        this.originalConfig = config || {};
    }
    build() {
        try {
            this.handleConfig(this.originalConfig);
            return this.effectiveConfig;
        }
        catch (err) {
            throw new core_1.InvalidConfigurationError('Config validation error: ' + err.message, err);
        }
    }
    handleConfig(jsonObject) {
        ConfigBuilder.validateSecurityConfiguration(jsonObject);
        for (const key in jsonObject) {
            const value = jsonObject[key];
            if (key === 'clusterName') {
                this.effectiveConfig.clusterName = (0, Util_1.tryGetString)(value);
            }
            else if (key === 'instanceName') {
                this.effectiveConfig.instanceName = (0, Util_1.tryGetString)(value);
            }
            else if (key === 'properties') {
                this.handleProperties(value);
            }
            else if (key === 'clientLabels') {
                this.handleClientLabels(value);
            }
            else if (key === 'network') {
                this.handleNetwork(value);
            }
            else if (key === 'connectionStrategy') {
                this.handleConnectionStrategy(value);
            }
            else if (key === 'lifecycleListeners') {
                this.handleLifecycleListeners(value);
            }
            else if (key === 'membershipListeners') {
                this.handleMembershipListeners(value);
            }
            else if (key === 'serialization') {
                this.handleSerialization(value);
            }
            else if (key === 'nearCaches') {
                this.handleNearCaches(value);
            }
            else if (key === 'reliableTopics') {
                this.handleReliableTopics(value);
            }
            else if (key === 'flakeIdGenerators') {
                this.handleFlakeIdGenerators(value);
            }
            else if (key === 'loadBalancer') {
                this.handleLoadBalancer(value);
            }
            else if (key === 'customLogger') {
                this.handleLogger(value);
            }
            else if (key === 'customCredentials') {
                this.handleCustomCredentials(value);
            }
            else if (key === 'backupAckToClientEnabled') {
                this.effectiveConfig.backupAckToClientEnabled = (0, Util_1.tryGetBoolean)(value);
            }
            else if (key === 'security') {
                this.handleSecurity(value);
            }
            else {
                throw new RangeError(`Unexpected config key '${key}' is passed to the Hazelcast Client`);
            }
        }
    }
    static validateSecurityConfiguration(jsonObject) {
        if ('security' in jsonObject && 'customCredentials' in jsonObject) {
            throw new RangeError('Ambiguous security configuration is found. ' +
                'Use one of \'security\' or \'customCredentials\' elements, not both.');
        }
    }
    handleSecurity(jsonObject) {
        let isCredentialsSet = false;
        for (const key in jsonObject) {
            if (isCredentialsSet) {
                throw new RangeError('Security configuration may only contain one of the supported credential types. ' +
                    'Multiple credential types are passed to the Hazelcast Client.');
            }
            const value = jsonObject[key];
            if (key === 'usernamePassword') {
                this.handleUsernamePasswordCredentials(value);
            }
            else if (key === 'token') {
                this.handleTokenCredentials(value);
            }
            else if (key === 'custom') {
                this.effectiveConfig.security.custom = value;
            }
            else {
                throw new RangeError(`Unexpected security config ${key} is passed to the Hazelcast Client`);
            }
            isCredentialsSet = true;
        }
    }
    handleUsernamePasswordCredentials(jsonObject) {
        let username = null;
        let password = null;
        for (const key in jsonObject) {
            const value = jsonObject[key];
            if (key === 'username') {
                username = (0, Util_1.tryGetStringOrNull)(value);
            }
            else if (key === 'password') {
                password = (0, Util_1.tryGetStringOrNull)(value);
            }
            else {
                throw new RangeError(`Unexpected username password credentials option '${key}' is passed to the Hazelcast Client`);
            }
        }
        this.effectiveConfig.security.usernamePassword = new security_1.UsernamePasswordCredentialsImpl(username, password);
    }
    handleTokenCredentials(jsonObject) {
        let token;
        let encoding;
        for (const key in jsonObject) {
            const value = jsonObject[key];
            if (key === 'token') {
                token = (0, Util_1.tryGetString)(value);
            }
            else if (key === 'encoding') {
                encoding = (0, Util_1.tryGetEnum)(security_1.TokenEncoding, value);
            }
            else {
                throw new RangeError(`Unexpected token credentials option '${key}' is passed to the Hazelcast Client`);
            }
        }
        if (token == null) {
            throw new RangeError('\'token\' option must be provided in token credentials.');
        }
        this.effectiveConfig.security.token = new security_1.TokenCredentialsImpl(token, encoding);
    }
    handleConnectionStrategy(jsonObject) {
        for (const key in jsonObject) {
            const value = jsonObject[key];
            if (key === 'asyncStart') {
                this.effectiveConfig.connectionStrategy.asyncStart = (0, Util_1.tryGetBoolean)(value);
            }
            else if (key === 'reconnectMode') {
                this.effectiveConfig.connectionStrategy.reconnectMode = (0, Util_1.tryGetEnum)(ConnectionStrategyConfig_1.ReconnectMode, value);
            }
            else if (key === 'connectionRetry') {
                this.handleConnectionRetry(value);
            }
            else {
                throw new RangeError(`Unexpected connection strategy config '${key}' is passed to the Hazelcast Client`);
            }
        }
    }
    handleConnectionRetry(jsonObject) {
        for (const key in jsonObject) {
            const value = jsonObject[key];
            if (key === 'initialBackoffMillis') {
                (0, Util_1.assertNonNegativeNumber)(value, 'Initial backoff must be non-negative!');
                this.effectiveConfig.connectionStrategy.connectionRetry.initialBackoffMillis = value;
            }
            else if (key === 'maxBackoffMillis') {
                (0, Util_1.assertNonNegativeNumber)(value, 'Max backoff must be non-negative!');
                this.effectiveConfig.connectionStrategy.connectionRetry.maxBackoffMillis = value;
            }
            else if (key === 'multiplier') {
                if (typeof value !== 'number' || value < 1.0) {
                    throw new RangeError('Multiplier must be a number that is greater than or equal to 1.0!');
                }
                this.effectiveConfig.connectionStrategy.connectionRetry.multiplier = value;
            }
            else if (key === 'clusterConnectTimeoutMillis') {
                if (typeof value !== 'number' || (value < 0 && value !== -1)) {
                    throw new RangeError('ClusterConnectTimeoutMillis can be only non-negative number or -1!');
                }
                this.effectiveConfig.connectionStrategy.connectionRetry.clusterConnectTimeoutMillis = value;
            }
            else if (key === 'jitter') {
                if (typeof value !== 'number' || (value < 0 || value > 1)) {
                    throw new RangeError('Jitter must be a number in range [0.0, 1.0]!');
                }
                this.effectiveConfig.connectionStrategy.connectionRetry.jitter = value;
            }
            else {
                throw new RangeError(`Unexpected connection retry config '${key}' is passed to the Hazelcast Client`);
            }
        }
    }
    handleClientLabels(jsonObject) {
        const labelsArray = (0, Util_1.tryGetArray)(jsonObject);
        for (const index in labelsArray) {
            const label = labelsArray[index];
            this.effectiveConfig.clientLabels.push(label);
        }
    }
    handleNetwork(jsonObject) {
        for (const key in jsonObject) {
            if (key === 'clusterMembers') {
                this.handleClusterMembers(jsonObject[key]);
            }
            else if (key === 'smartRouting') {
                this.effectiveConfig.network.smartRouting = (0, Util_1.tryGetBoolean)(jsonObject[key]);
            }
            else if (key === 'redoOperation') {
                this.effectiveConfig.network.redoOperation = (0, Util_1.tryGetBoolean)(jsonObject[key]);
            }
            else if (key === 'connectionTimeout') {
                this.effectiveConfig.network.connectionTimeout = (0, Util_1.tryGetNumber)(jsonObject[key]);
            }
            else if (key === 'ssl') {
                this.handleSSL(jsonObject[key]);
            }
            else if (key === 'hazelcastCloud') {
                this.handleHazelcastCloud(jsonObject[key]);
            }
            else {
                throw new RangeError(`Unexpected network option '${key}' is passed to the Hazelcast Client`);
            }
        }
    }
    handleHazelcastCloud(jsonObject) {
        for (const key in jsonObject) {
            if (key === 'discoveryToken') {
                this.effectiveConfig.network.hazelcastCloud.discoveryToken = (0, Util_1.tryGetString)(jsonObject[key]);
            }
            else {
                throw new RangeError(`Unexpected hazelcast cloud option '${key}' is passed to the Hazelcast Client`);
            }
        }
    }
    parseProperties(jsonObject) {
        const props = {};
        for (const key in jsonObject) {
            props[key] = jsonObject[key];
        }
        return props;
    }
    handleSSL(jsonObject) {
        const sslConfigKeys = new Set(Object.keys(this.effectiveConfig.network.ssl));
        for (const key in jsonObject) {
            if (!sslConfigKeys.has(key)) {
                throw new RangeError(`Unexpected ssl option '${key}' is passed to the Hazelcast Client`);
            }
        }
        const sslEnabled = (0, Util_1.tryGetBoolean)(jsonObject.enabled);
        this.effectiveConfig.network.ssl.enabled = sslEnabled;
        if (jsonObject.sslOptions) {
            if (jsonObject.sslOptionsFactory || jsonObject.sslOptionsFactoryProperties) {
                throw new RangeError('Invalid configuration. Either SSL options should be set manually or SSL factory'
                    + ' should be used.');
            }
            this.effectiveConfig.network.ssl.sslOptions = jsonObject.sslOptions;
        }
        else if (jsonObject.sslOptionsFactory || jsonObject.sslOptionsFactoryProperties) {
            this.effectiveConfig.network.ssl.sslOptionsFactory = jsonObject.sslOptionsFactory;
            this.effectiveConfig.network.ssl.sslOptionsFactoryProperties = jsonObject.sslOptionsFactoryProperties
                ? this.parseProperties(jsonObject.sslOptionsFactoryProperties) : null;
        }
    }
    handleClusterMembers(jsonObject) {
        const addressArray = (0, Util_1.tryGetArray)(jsonObject);
        for (const index in addressArray) {
            const address = addressArray[index];
            this.effectiveConfig.network.clusterMembers.push((0, Util_1.tryGetString)(address));
        }
    }
    handleProperties(jsonObject) {
        for (const key in jsonObject) {
            let value = jsonObject[key];
            try {
                switch (key) {
                    case 'hazelcast.client.heartbeat.interval':
                        value = (0, Util_1.tryGetNumber)(value);
                        break;
                    case 'hazelcast.client.heartbeat.timeout':
                        value = (0, Util_1.tryGetNumber)(value);
                        break;
                    case 'hazelcast.client.invocation.retry.pause.millis':
                        value = (0, Util_1.tryGetNumber)(value);
                        break;
                    case 'hazelcast.client.invocation.timeout.millis':
                        value = (0, Util_1.tryGetNumber)(value);
                        break;
                    case 'hazelcast.client.internal.clean.resources.millis':
                        value = (0, Util_1.tryGetNumber)(value);
                        break;
                    case 'hazelcast.client.cloud.url':
                        value = (0, Util_1.tryGetString)(value);
                        break;
                    case 'hazelcast.client.statistics.enabled':
                        value = (0, Util_1.tryGetBoolean)(value);
                        break;
                    case 'hazelcast.client.statistics.period.seconds':
                        value = (0, Util_1.tryGetNumber)(value);
                        break;
                    case 'hazelcast.invalidation.reconciliation.interval.seconds':
                        value = (0, Util_1.tryGetNumber)(value);
                        break;
                    case 'hazelcast.invalidation.max.tolerated.miss.count':
                        value = (0, Util_1.tryGetNumber)(value);
                        break;
                    case 'hazelcast.invalidation.min.reconciliation.interval.seconds':
                        value = (0, Util_1.tryGetNumber)(value);
                        break;
                    case 'hazelcast.logging.level':
                        (0, Util_1.tryGetEnum)(logging_1.LogLevel, value);
                        break;
                    case 'hazelcast.client.autopipelining.enabled':
                        value = (0, Util_1.tryGetBoolean)(value);
                        break;
                    case 'hazelcast.client.autopipelining.threshold.bytes':
                        value = (0, Util_1.tryGetNumber)(value);
                        break;
                    case 'hazelcast.client.socket.no.delay':
                        value = (0, Util_1.tryGetBoolean)(value);
                        break;
                    case 'hazelcast.client.shuffle.member.list':
                        value = (0, Util_1.tryGetBoolean)(value);
                        break;
                    case 'hazelcast.client.operation.backup.timeout.millis':
                        value = (0, Util_1.tryGetNumber)(value);
                        break;
                    case 'hazelcast.client.operation.fail.on.indeterminate.state':
                        value = (0, Util_1.tryGetBoolean)(value);
                        break;
                    case 'hazelcast.discovery.public.ip.enabled':
                        if (value !== null && typeof value !== 'boolean') {
                            throw new RangeError(`${value} is not null or a boolean.`);
                        }
                        break;
                    default:
                        throw new RangeError(`Unexpected property '${key}' is passed to the Hazelcast Client`);
                }
            }
            catch (e) {
                throw new RangeError(`Property validation error: Property: ${key}, value: ${value}. Error: ${e}`);
            }
            this.effectiveConfig.properties[key] = value;
        }
    }
    handleLifecycleListeners(jsonObject) {
        const listenersArray = (0, Util_1.tryGetArray)(jsonObject);
        for (const listener of listenersArray) {
            this.effectiveConfig.lifecycleListeners.push(listener);
        }
    }
    handleMembershipListeners(jsonObject) {
        const listenersArray = (0, Util_1.tryGetArray)(jsonObject);
        for (const listener of listenersArray) {
            this.effectiveConfig.membershipListeners.push(listener);
        }
    }
    handleSerialization(jsonObject) {
        for (const key in jsonObject) {
            if (key === 'defaultNumberType') {
                this.effectiveConfig.serialization.defaultNumberType = (0, Util_1.tryGetString)(jsonObject[key]).toLowerCase();
            }
            else if (key === 'isBigEndian') {
                this.effectiveConfig.serialization.isBigEndian = (0, Util_1.tryGetBoolean)(jsonObject[key]);
            }
            else if (key === 'portableVersion') {
                this.effectiveConfig.serialization.portableVersion = (0, Util_1.tryGetNumber)(jsonObject[key]);
            }
            else if (key === 'dataSerializableFactories') {
                for (const index in jsonObject[key]) {
                    const idx = Number.parseInt(index);
                    this.effectiveConfig.serialization
                        .dataSerializableFactories[idx] = jsonObject[key][index];
                }
            }
            else if (key === 'portableFactories') {
                for (const index in jsonObject[key]) {
                    const idx = Number.parseInt(index);
                    this.effectiveConfig.serialization
                        .portableFactories[idx] = jsonObject[key][index];
                }
            }
            else if (key === 'globalSerializer') {
                const globalSerializer = jsonObject[key];
                this.effectiveConfig.serialization.globalSerializer = globalSerializer;
            }
            else if (key === 'customSerializers') {
                this.handleCustomSerializers(jsonObject[key]);
            }
            else if (key === 'jsonStringDeserializationPolicy') {
                this.effectiveConfig.serialization
                    .jsonStringDeserializationPolicy = (0, Util_1.tryGetEnum)(JsonStringDeserializationPolicy_1.JsonStringDeserializationPolicy, jsonObject[key]);
            }
            else {
                throw new RangeError(`Unexpected serialization config '${key}' is passed to the Hazelcast Client`);
            }
        }
    }
    handleCustomSerializers(jsonObject) {
        const serializersArray = (0, Util_1.tryGetArray)(jsonObject);
        for (const serializer of serializersArray) {
            this.effectiveConfig.serialization.customSerializers.push(serializer);
        }
    }
    handleNearCaches(jsonObject) {
        for (const name in jsonObject) {
            const ncConfig = jsonObject[name];
            const nearCacheConfig = new NearCacheConfig_1.NearCacheConfigImpl();
            nearCacheConfig.name = name;
            for (const key in ncConfig) {
                if (key === 'invalidateOnChange') {
                    nearCacheConfig.invalidateOnChange = (0, Util_1.tryGetBoolean)(ncConfig[key]);
                }
                else if (key === 'maxIdleSeconds') {
                    nearCacheConfig.maxIdleSeconds = (0, Util_1.tryGetNumber)(ncConfig[key]);
                }
                else if (key === 'inMemoryFormat') {
                    nearCacheConfig.inMemoryFormat = (0, Util_1.tryGetEnum)(InMemoryFormat_1.InMemoryFormat, ncConfig[key]);
                }
                else if (key === 'timeToLiveSeconds') {
                    nearCacheConfig.timeToLiveSeconds = (0, Util_1.tryGetNumber)(ncConfig[key]);
                }
                else if (key === 'evictionPolicy') {
                    nearCacheConfig.evictionPolicy = (0, Util_1.tryGetEnum)(EvictionPolicy_1.EvictionPolicy, ncConfig[key]);
                }
                else if (key === 'evictionMaxSize') {
                    nearCacheConfig.evictionMaxSize = (0, Util_1.tryGetNumber)(ncConfig[key]);
                }
                else if (key === 'evictionSamplingCount') {
                    nearCacheConfig.evictionSamplingCount = (0, Util_1.tryGetNumber)(ncConfig[key]);
                }
                else if (key === 'evictionSamplingPoolSize') {
                    nearCacheConfig.evictionSamplingPoolSize = (0, Util_1.tryGetNumber)(ncConfig[key]);
                }
                else {
                    throw new RangeError(`Unexpected near cache config '${key}' for near cache ${name}`
                        + 'is passed to the Hazelcast Client');
                }
            }
            this.effectiveConfig.nearCaches[nearCacheConfig.name] = nearCacheConfig;
        }
    }
    handleReliableTopics(jsonObject) {
        for (const name in jsonObject) {
            const jsonRtCfg = jsonObject[name];
            const reliableTopicConfig = new ReliableTopicConfig_1.ReliableTopicConfigImpl();
            reliableTopicConfig.name = name;
            for (const key in jsonRtCfg) {
                if (key === 'readBatchSize') {
                    reliableTopicConfig.readBatchSize = jsonRtCfg[key];
                }
                else if (key === 'overloadPolicy') {
                    reliableTopicConfig.overloadPolicy = (0, Util_1.tryGetEnum)(proxy_1.TopicOverloadPolicy, jsonRtCfg[key]);
                }
                else {
                    throw new RangeError(`Unexpected reliable topic config '${key}' for reliable topic ${name}`
                        + 'is passed to the Hazelcast Client');
                }
            }
            this.effectiveConfig.reliableTopics[reliableTopicConfig.name] = reliableTopicConfig;
        }
    }
    handleFlakeIdGenerators(jsonObject) {
        for (const name in jsonObject) {
            const fidConfig = jsonObject[name];
            const flakeIdConfig = new FlakeIdGeneratorConfig_1.FlakeIdGeneratorConfigImpl();
            flakeIdConfig.name = name;
            for (const key in fidConfig) {
                if (key === 'prefetchCount') {
                    flakeIdConfig.prefetchCount = (0, Util_1.tryGetNumber)(fidConfig[key]);
                }
                else if (key === 'prefetchValidityMillis') {
                    flakeIdConfig.prefetchValidityMillis = (0, Util_1.tryGetNumber)(fidConfig[key]);
                }
                else {
                    throw new RangeError(`Unexpected flake id generator config '${key}' for flake id generator ${name}`
                        + 'is passed to the Hazelcast Client');
                }
            }
            this.effectiveConfig.flakeIdGenerators[flakeIdConfig.name] = flakeIdConfig;
        }
    }
    handleLoadBalancer(jsonObject) {
        for (const key in jsonObject) {
            if (key === 'type') {
                this.effectiveConfig.loadBalancer.type = (0, Util_1.tryGetEnum)(LoadBalancerConfig_1.LoadBalancerType, jsonObject[key]);
            }
            else if (key === 'customLoadBalancer') {
                this.effectiveConfig.loadBalancer.customLoadBalancer = jsonObject[key];
            }
            else {
                throw new RangeError(`Unexpected load balancer config '${key}' is passed to the Hazelcast Client`);
            }
        }
    }
    handleLogger(jsonObject) {
        this.effectiveConfig.customLogger = jsonObject;
    }
    handleCustomCredentials(jsonObject) {
        this.effectiveConfig.customCredentials = jsonObject;
    }
}
exports.ConfigBuilder = ConfigBuilder;
//# sourceMappingURL=ConfigBuilder.js.map